from .heart_disease_rf import get_heart_disease_rf
from .iris_lda import get_iris_test
from .ames_housing_gbm import get_ames_gbm_test
from .boston_gbm import get_boston_gbm_test
from .titanic_rf import get_titanic_rf
import data